library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.my_package.all;

entity data_path is
port (
	a, b, f: in std_logic_vector(n downto 0);
	u, v: in std_logic_vector(n-1 downto 0);
	d, p: in STD_LOGIC_VECTOR (logN downto 0);
	new_a, new_b: out std_logic_vector(n downto 0);
	new_u, new_v: out std_logic_vector(n-1 downto 0);
	new_d, new_p: out STD_LOGIC_VECTOR (logN downto 0)
);
end data_path;

architecture circuit of data_path is

component UV_cell
port (
	u, v: in STD_LOGIC_VECTOR (N-1 downto 0);
	f: in STD_LOGIC_VECTOR (N downto 0);
	cont_u: in std_logic_vector(1 downto 0);
	c_ce_v: in std_logic;
	new_u, new_v: out STD_LOGIC_VECTOR (N-1 downto 0)
);
end component;

component AB_cell
port (
	a, b: in STD_LOGIC_VECTOR (N downto 0);
	oper_a, c_ce_b: in std_logic;
	new_a, new_b: out STD_LOGIC_VECTOR (N downto 0)
);
end component;

COMPONENT control_ruta
PORT(
	d,p : IN std_logic_vector(logN downto 0);
	a_0, b_0, v_0, u_0 : IN std_logic;          
	c_ce_b, c_ce_v, c_ce_p: OUT std_logic;
	cont_u, cont_d: OUT std_logic_vector(1 downto 0)		
	);
END COMPONENT;

signal cont_d, cont_u: std_logic_vector(1 downto 0);
signal oper_a, c_ce_b, c_ce_v, c_ce_d, c_ce_p: std_logic;

begin

uv_block: uv_cell port map (u, v, f, cont_u, c_ce_v, new_u, new_v);
ab_block: ab_cell port map (a, b, oper_a, c_ce_b, new_a, new_b);
control_dp: control_ruta PORT MAP(
		d => d, p => p,
		a_0 => a(0), b_0 => b(0), v_0 => v(0), u_0 => u(0),
		c_ce_b => c_ce_b,	c_ce_v => c_ce_v,	c_ce_p => c_ce_p,	
		cont_u => cont_u,	cont_d => cont_d
	);

oper_a <= cont_u(0);

new_p <= p-1 when c_ce_p = '1' else p;

with cont_d select new_d <= not(d) when "11", 
                            d - 1 when "10",
									 d when others;

end circuit;

