---------------------------------------------------------------------
-- Divider over GF(2**m). Binary algorithm
---------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.my_package.all;

entity divider is
port (
num, den, f_in: in std_logic_vector(N-1 downto 0);
clk, reset, start: in std_logic;
done: out std_logic;
z: out std_logic_vector(N-1 downto 0)
);
end divider;

architecture circuit of divider is

	COMPONENT data_path
	PORT(
		a, b, f : IN std_logic_vector(N downto 0);
		u, v : IN std_logic_vector(N-1 downto 0);
		d, p : IN std_logic_vector(logN downto 0);          
		new_a, new_b : OUT std_logic_vector(N downto 0);
		new_u, new_v : OUT std_logic_vector(N-1 downto 0);
		new_d, new_p : OUT std_logic_vector(logN downto 0)
		);
	END COMPONENT;

signal p_gt_0, first_step, capture: std_logic; --, oper_a, ce_a, ce_b, ce_u, ce_v, ce_d, ce_p: std_logic;
signal boolean_sum_p: std_logic_vector(logN-1 downto 0);
signal a, b, i_a, i_b, new_a, new_b, f : std_logic_vector(n downto 0);
signal u, v, i_u, i_v, new_u, new_v: std_logic_vector(n-1 downto 0);
signal d, new_d, i_d, p, new_p, i_p: std_logic_vector(logN downto 0);

signal n_a1, n_b1: std_logic_vector(n downto 0);
signal n_u1, n_v1: std_logic_vector(n-1 downto 0);
signal n_d1, n_p1: std_logic_vector(logN downto 0);
signal n_a2, n_b2: std_logic_vector(n downto 0);
signal n_u2, n_v2: std_logic_vector(n-1 downto 0);
signal n_d2, n_p2: std_logic_vector(logN downto 0);

signal current_state: std_logic_vector(1 downto 0);

begin

data_path_block: data_path PORT MAP(
		a => a, b => b, f => f,
		u => u, v => v,
		d => d, p => p,
		new_a => new_a, new_b => new_b,
		new_u => new_u, new_v => new_v,
		new_d => new_d, new_p => new_p
	);


with a(0) select z <= v when '0', u when others;

--mux for 1st step
with first_step select i_a <= new_a when '0', ('0' & den) when others;
with first_step select i_u <= new_u when '0', num when others;
with first_step select i_b <= new_b when '0', f when others;
with first_step select i_v <= new_v when '0', (others => '0') when others;
--with first_step select i_p <= new_p when '0', n_minus_1 when others;
with first_step select i_p <= new_p when '0', conv_std_logic_vector(n-2, logN+1) when others;
with first_step select i_d <= new_d when '0', (others => '1') when others;

process(clk, reset)
begin
	if reset = '1' then 
	   a <= (others => '0'); b <= (others => '0');
	   u <= (others => '0'); v <= (others => '0');
	   d <= (others => '0'); p <= (others => '0');
	elsif clk'event and clk = '1' then
	  if capture = '1' then
		a <= i_a; b <= i_b;
		u <= i_u; v <= i_v;
		d <= i_d; p <= i_p;
	  end if;
	end if;
end process;

process(clk, reset)
begin
	if reset = '1' then 
	   f <= (others => '0'); 
	elsif clk'event and clk = '1' then
	  if first_step = '1' then
		 f <= '1' & f_in;
	  end if;
	end if;
end process;

boolean_sum_p(0) <= p(0);
p_gt_0 <= not(p(logN));

--state transitions
process(clk, reset)
begin
if reset = '1' then current_state <= "00"; 
elsif clk'event and clk = '1' then 
	case current_state is
	when "00" => if start = '0' then current_state <= current_state + 1; end if; 
	when "01" => if start = '1' then current_state <= current_state + 1; end if;
	when "10" => current_state <= current_state + 1;
	when others => if p_gt_0 = '0' then current_state <= current_state + 1; end if;
	end case;
end if;
end process;

--control signals
process(current_state, p_gt_0)
begin
   case current_state is
		when "00"  => first_step <= '0'; done <= '1'; capture <= '0';
		when "01" => first_step <= '0'; done <= '1'; capture <= '0';
		when "10" => first_step <= '1'; done <= '1'; capture <= '1';
		when others =>	 first_step <= '0'; done <= '0'; 
		                if p_gt_0 = '0' then capture <= '0'; else capture <= '1'; end if;
   end case;																				  
end process;

end circuit;